/*******************************************************************************
* This software program is available to you under a choice of one of two 
* licenses. You may choose to be licensed under either the GNU General Public 
* License (GPL) Version 2, June 1991, available at 
* http://www.fsf.org/copyleft/gpl.html, or the Intel BSD + Patent License, the 
* text of which follows:
* 
* Recipient has requested a license and Intel Corporation ("Intel") is willing
* to grant a license for the software entitled Linux Intel Advanced Network
* Services (iANS) (the "Licensed Program") being provided by Intel Corporation.
* The following definitions apply to this license:
* 
* "Licensed Patents" means patent claims licensable by Intel Corporation which 
* are necessarily infringed by the use of sale of the Software alone or when 
* combined with the operating system referred to below.
* 
* "Recipient" means the party to whom Intel delivers this Software.
* 
* "Licensee" means Recipient and those third parties that receive a license to 
* any operating system available under the GNU Public License version 2.0 or 
* later.
* 
* Copyright (c) 1999 - 2002 Intel Corporation.
* All rights reserved.
* 
* The license is provided to Recipient and Recipient's Licensees under the 
* following terms.
* 
* Redistribution and use in source and binary forms of the Software, with or 
* without modification, are permitted provided that the following conditions 
* are met:
* 
* Redistributions of source code of the Software may retain the above 
* copyright notice, this list of conditions and the following disclaimer.
* 
* Redistributions in binary form of the Software may reproduce the above 
* copyright notice, this list of conditions and the following disclaimer in 
* the documentation and/or materials provided with the distribution.
* 
* Neither the name of Intel Corporation nor the names of its contributors 
* shall be used to endorse or promote products derived from this Software 
* without specific prior written permission.
* 
* Intel hereby grants Recipient and Licensees a non-exclusive, worldwide, 
* royalty-free patent license under Licensed Patents to make, use, sell, offer 
* to sell, import and otherwise transfer the Software, if any, in source code 
* and object code form. This license shall include changes to the Software 
* that are error corrections or other minor changes to the Software that do 
* not add functionality or features when the Software is incorporated in any 
* version of an operating system that has been distributed under the GNU 
* General Public License 2.0 or later. This patent license shall apply to the 
* combination of the Software and any operating system licensed under the GNU 
* Public License version 2.0 or later if, at the time Intel provides the 
* Software to Recipient, such addition of the Software to the then publicly 
* available versions of such operating systems available under the GNU Public 
* License version 2.0 or later (whether in gold, beta or alpha form) causes 
* such combination to be covered by the Licensed Patents. The patent license 
* shall not apply to any other combinations which include the Software. NO 
* hardware per se is licensed hereunder.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MECHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR IT CONTRIBUTORS BE LIABLE FOR ANY 
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
* ANY LOSS OF USE; DATA, OR PROFITS; OR BUSINESS INTERUPTION) HOWEVER CAUSED 
* AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY OR 
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/
/***************************************************************************
**                                                                        **
** INTEL CORPORATION                                                      **
**                                                                        **
** This software is supplied under the terms of the license included      **
** above.  All use of this software must be in accordance with the terms  **
** of that license.                                                       **
**                                                                        **
**  Module Name:                                                          **
**    ians_kernel.c                                                       **
**                                                                        **
**  Abstract:                                                             **
**    This module contains all the functions needed for the interface of  **
**    the iANS module with the kernel.                                    **
**                                                                        **
**  Environment:                                                          **
**    Kernel Mode (linux 2.2.x, 2.4.x)                                    **
**                                                                        **
***************************************************************************/

#define _IANS_MAIN_MODULE_C_

#include <linux/module.h>
#include <linux/sched.h>
#include <linux/netdevice.h>
#include <linux/init.h>
#include <linux/rtnetlink.h>
#include <linux/pkt_sched.h>
//#include <linux/config.h>
//#include <linux/kernel.h>
//#include <linux/types.h>
//#include <linux/fcntl.h>
//#include <linux/interrupt.h>
//#include <linux/ptrace.h>
//#include <linux/ioport.h>
//#include <linux/in.h>
//#include <linux/malloc.h>
//#include <linux/malloc.h>
//#include <linux/etherdevice.h>
//#include <linux/skbuff.h>
//#include <linux/version.h>
//#include <linux/param.h>
//#include <asm/system.h>
//#include <asm/bitops.h>
//#include <asm/io.h>
//#include <asm/dma.h>

#ifdef LINUX_KERNEL_24X
#include <linux/spinlock.h>
#include <linux/notifier.h>
//#include <linux/if.h>
#endif

#include "ians_kcompat.h"
#include "ians_kernel.h"
#include "ans_interface.h"
#include "incg_open_utils.h"
#include "ians_base.h"
#include "incg_gp_mem.h"
#include "base_comm_os.h"
#include "pubDefs.h"
//#include "base_comm.h"
//#include "incg_net.h"
//#include "incg_flow.h"
//#include "status.h"


#define DEST_ADDR_PTR(osMsg, flow)  \
	(flow == TRANSMIT_FLOW ? (MAC_ADDR *)(osMsg->data) : \
							(MAC_ADDR *)(osMsg->mac.ethernet->h_dest));

#define SRC_ADDR_PTR(osMsg, flow) \
	(flow == TRANSMIT_FLOW ? ((MAC_ADDR *)(osMsg->data) + 1) :\
							(MAC_ADDR *)(osMsg->mac.ethernet->h_source));

#define LENGTH_TYPE_PTR(osMsg, flow) \
	(flow == TRANSMIT_FLOW ? (USHORT *)((MAC_ADDR *)(osMsg->data) + 2) :\
							(USHORT *)(&(osMsg->mac.ethernet->h_proto)));


static struct net_device *control_g = NULL;

#ifdef LINUX_KERNEL_24X
static spinlock_t ians_lock, ians_ind_lock;
#endif

static int iansVadapterOpen(struct net_device *dev);
static int iansVadapterStop(struct net_device *dev );
static int iansVadapterSend(struct sk_buff *skb, struct net_device *dev);
static struct net_device_stats *iansVadapterGetStats(struct net_device *dev); 
static void iansVadapterSetMulticastList(struct net_device *dev);
static int iansVadapterSetMacAddress(struct net_device *dev, void *addr);
static int iansVadapterDoIoctl(struct net_device *dev, struct ifreq *rq, int cmd);
static int iansVadapterChangeMtu(struct net_device *dev, int new_mtu);

static int iansOpenReceive(struct sk_buff *skb, struct net_device *dev, struct packet_type *pt);

static struct packet_type iansPacketType =
{
      IANS_FRAME_TYPE,  // Note : no need for htons !!!
      NULL,             // Null = All devices 
      iansOpenReceive,  // receive function
      NULL,             // Private to the packet type          
      NULL,             // next
};

//--------------------- CONVERT OS TYPES TO ANS TYPES -----------------------
static inline int _convertOsMCListToAnsMCList(struct dev_mc_list *osList, int osCount, MULTICAST_LIST *ansList)
{

      struct dev_mc_list *currList;
      int res;

      ASSERT(osList && ansList);

      for (currList = osList; currList != NULL; currList = currList->next) {
            res = ANSAddMulticastAddress (ansList,
                                          (MAC_ADDR*)currList->dmi_addr,
                                          currList->dmi_addrlen,
                                          currList->dmi_users,
                                          currList->dmi_gusers,
                                          NULL);

            if (res != FLOW_OK) {
                  if (res == FLOW_GP_RESOURCES) {
                        IANS_PRINT_ERROR("kmalloc failed.\n");
                  }
                  IANS_PRINT_ERROR("Failed in adding a multicast address.\n");
                  return IANS_ERROR;
            }
      }

      ASSERT(osCount == ansList->NumAddresses);

      return IANS_OK;
}


static inline void _convertOsPFToAnsPF(USHORT osFlags, PACKET_FILTER *packetFilter)
{
      if (osFlags & IFF_PROMISC) {
            *packetFilter |= PF_PROMISCUOUS_BIT;
      }
      else {
            *packetFilter &= ~(PF_PROMISCUOUS_BIT);
      }

      if (osFlags & IFF_ALLMULTI) {
            *packetFilter |= PF_ALLMCA_BIT;
      }
      else {
            *packetFilter &= ~(PF_ALLMCA_BIT);
      }
}


static inline void _convertOsMessageToAnsMessage(pOsMessage_t osMsg, message_t *ansMsg, IANS_FLOW_MODE flow)
{
      ASSERT(osMsg && ansMsg);

      ansMsg->pOsMessage = osMsg;
      ansMsg->pProtocol = &(osMsg->protocol);  
      ansMsg->Head = osMsg->head;                 
      ansMsg->Data = osMsg->data;
      
      if (flow == TRANSMIT_FLOW)
            ansMsg->AttrHeader = ansMsg->AttrHeader = (char *)iANSGetTransmitAttributeHeader(osMsg); 
      else
            ansMsg->AttrHeader = ansMsg->AttrHeader = (char *)iANSGetReceiveAttributeHeader(osMsg); 

      ansMsg->LocalAttr = NULL;
      ansMsg->Len = osMsg->tail - osMsg->data;
      
      //IANS_PRINT_ERROR("skb->priority = %d\n", osMsg->priority);
      ansMsg->Priority = &(osMsg->priority);

      ansMsg->pDest = DEST_ADDR_PTR(osMsg, flow);
      ansMsg->pSrc = SRC_ADDR_PTR(osMsg, flow); 
      ansMsg->pLengthType = LENGTH_TYPE_PTR(osMsg, flow);

}


//------- (OS/ANS) NET DEVICE ALLOCATION AND INITIALIZATION FUNCTIONS --------

struct net_device *iansDevAlloc(const char *name)
{
      int allocSize;
      struct net_device *dev;
#ifdef LINUX_KERNEL_24X
      struct net_device *temp;
#endif

#ifdef LINUX_KERNEL_22X
      if (dev_get_by_name( name) != NULL) {
#else
      if ((temp = dev_get_by_name(name)) != NULL) {
            dev_put(temp);
#endif
            IANS_PRINT_ERROR("A device with this name (%s) already exists\n", name);
            return NULL;
      }

#ifdef LINUX_KERNEL_22X
      allocSize = sizeof(struct net_device)+IFNAMSIZ;
#else
      allocSize = sizeof(struct net_device);
#endif	
      allocSize += 3;   
      allocSize &= ~3;

      iNCGGPSizedMemAlloc ((void **)(&dev) , allocSize);
      if (dev == NULL) {
            IANS_PRINT_ERROR ("kmalloc failed.\n");
            return NULL;
      }

      memset( dev , 0 , allocSize );

#ifdef LINUX_KERNEL_22X
      dev->name=(char *)(dev+1);
#endif

      iAnsStrCpy(dev->name,name);

      return dev;
}


static struct net_device *iansInitEtherdev(const char *name)
{
      struct net_device *dev = NULL;

      if ((  dev = iansDevAlloc ( name ) ) == NULL)
            return NULL;

      ether_setup(dev);       

      return dev;
}


int iansInitVadapterDev(const char *name, device_t **ansVadapterDev)
{
      struct net_device  *osVadapterDev;

      ASSERT(name);

      if (( osVadapterDev = iansInitEtherdev( name ) ) == NULL) {
            return IANS_VADAPTER_INITIALIZE_FAIL;
      }

      osVadapterDev->open               = iansVadapterOpen;
      osVadapterDev->stop               = iansVadapterStop;
      osVadapterDev->hard_start_xmit    = iansVadapterSend;
      osVadapterDev->get_stats          = iansVadapterGetStats;
      osVadapterDev->set_multicast_list = iansVadapterSetMulticastList;
      osVadapterDev->set_mac_address    = iansVadapterSetMacAddress;
      osVadapterDev->do_ioctl           = iansVadapterDoIoctl;
      osVadapterDev->change_mtu         = iansVadapterChangeMtu;
#ifdef LINUX_KERNEL_22X
      osVadapterDev->start              = 0;
#endif
      osVadapterDev->tx_queue_len       = 100;

      iNCGGPSizedMemAlloc ((void **)(ansVadapterDev), sizeof(device_t) ); 
      if (*ansVadapterDev == NULL) {
            IANS_PRINT_ERROR("Failed to allocate an IANS device structure\n");
            iNCGGPMemFree( (void **)(ansVadapterDev) );
            return IANS_MEM_ERROR;
      }

      (*ansVadapterDev)->pOsDev = osVadapterDev;   
      (*ansVadapterDev)->name = osVadapterDev->name;
      (*ansVadapterDev)->mtu = &(osVadapterDev->mtu);
      (*ansVadapterDev)->mac = (MAC_ADDR *)(osVadapterDev->dev_addr);

      return IANS_OK;
}


int iansInitMemberDev(const char *baseName, device_t **ansDev)
{
      struct net_device *dev;

      if ((dev = dev_get_by_name( baseName ) ) == NULL) {
            IANS_PRINT_ERROR("Device %s doesn't exist!.\n", baseName);
            return 	IANS_MEMBER_NAME_WRONG;
      }

      iNCGGPSizedMemAlloc ((void **)(ansDev), sizeof(device_t));
      if (*ansDev == NULL) {
            IANS_PRINT_ERROR("Failed to allocate an IANS device structure for a member.\n");
            return IANS_MEM_ERROR;
      }

      (*ansDev)->pOsDev             = dev;   
      (*ansDev)->name               = dev->name;
      (*ansDev)->tbusy              = FALSE;
      (*ansDev)->mac                = (MAC_ADDR *)(dev->dev_addr);
      (*ansDev)->mtu                = &(dev->mtu);
      (*ansDev)->open               = baseOpenDevice;
      (*ansDev)->close              = baseCloseDevice;
      (*ansDev)->send               = (int (*) (void *, void *) ) baseSend;
      (*ansDev)->getStats           = baseGetStats;
      (*ansDev)->setMulticastList   = baseSetMulticastList;
      (*ansDev)->setMacAddr         = baseSetMacAddr;
      (*ansDev)->doIoctl            = baseDoIoctl; 
      (*ansDev)->changeMtu          = baseChangeMtu;

      return IANS_OK;
}


int iansRegisterDev(device_t *ansDev)
{
      struct net_device * dev;
      BOOLEAN ioctlContext = ANSGetConfigExecuteCommand();  

      ASSERT(ansDev);
      ASSERT(ansDev->name);

      if (!ioctlContext) {
            IANS_PRINT_ERROR("Not in ioctl context!!!\n");
      }

      dev = (struct net_device *) GET_OS_DEV(ansDev);

      if (dev == NULL) {
            return IANS_VADAPTER_REGISTER_FAIL;
      }
      
      //IANS_PRINT_DEBUG("Registering vadapter %s\n", dev->name);

      register_netdevice(dev);

      return IANS_OK;
}

void iansUnregisterDev(device_t *ansDev,BOOLEAN close)
{
      struct net_device * devExist = NULL;
      BOOLEAN ioctlContext = ANSGetConfigExecuteCommand();

      ASSERT(ansDev);
      ASSERT(ansDev->name);

      if ((devExist = dev_get_by_name(ansDev->name))) {
#ifdef LINUX_KERNEL_24X
            dev_put(devExist);
#endif
            if (devExist != (struct net_device *) GET_OS_DEV(ansDev)) {
                  IANS_PRINT_ERROR("Mismatch of kernel and ANS pointers to %s's device structure - cann't unregister the net device!\n", ansDev->name);
                  return;
            }
            else {//unregister the device.
                  if (close) {
                        dev_close ( devExist );
                  }

                  (ioctlContext ? unregister_netdevice(devExist) : unregister_netdev(devExist));
            }
      }
      else {
            IANS_PRINT_ERROR("Device %s doesn't exist - cann't unregister the net device!\n", ansDev->name);
      }
}


// ---------------------- VADAPTER OS NET DEVICE FUNCTIONS ------------------------

static int iansVadapterOpen(struct net_device *dev)
{
      device_t *ansDev = NULL;
      int res = 0;
      int ansRes = IANS_OK;

      iansStartBH(&ians_lock);

      ansDev = ANSGetVadapterAnsDevice(dev->name);
      if (ansDev == NULL) {
            IANS_PRINT_ERROR("there is no such ans device.\n");
            return -EAGAIN;
      }

      ansRes = ansDev->open(ansDev);
      if (ansRes == IANS_OK) {
            MOD_INC_USE_COUNT;
#ifdef LINUX_KERNEL_22X
            dev->start = 1;
#endif      
      }
      else {
            res = -EPERM; //operation not permitted 
      }

      iansEndBH(&ians_lock);

      return res;
}


static int iansVadapterStop(struct net_device *dev)
{
      device_t *ansDev = NULL; 
      int ansRes = IANS_OK;

      iansStartBH(&ians_lock);

      ansDev = ANSGetVadapterAnsDevice(dev->name);
      if (ansDev == NULL) {
            IANS_PRINT_ERROR("there is no such ans device.\n");
            return -EAGAIN;
      }

      ansRes = ansDev->close(ansDev);
      if (ansRes == IANS_OK) {
            MOD_DEC_USE_COUNT;
#ifdef LINUX_KERNEL_22X
            dev->start = 0;
#endif
      }

      iansEndBH(&ians_lock);

      return 0;
}


static int iansVadapterSend(struct sk_buff *skb, struct net_device *dev)
{
      device_t *ansDev;
      static message_t msg;  //automatic variable to save allocation time. 
      int res = 0;
      int ansRes = IANS_OK;

      ASSERT(skb && dev);

      ASSERT(dev->name);

      iansSpinLock(&ians_lock);

      ansDev = ANSGetVadapterAnsDevice(dev->name);
      if (ansDev == NULL) {
            IANS_PRINT_ERROR("there is no such ans device.\n");
            return -EAGAIN;
      }

      ASSERT(((struct net_device *) GET_OS_DEV(ansDev)) == dev);
	  
	  iAnsMemSet( &msg, 0 , sizeof(message_t) );

      _convertOsMessageToAnsMessage(skb, &msg, TRANSMIT_FLOW);

      ansRes = ansDev->send(&msg, ansDev);
      if (ansRes != IANS_OK) {
            res = 1; //The base driver return value in case of failure
      }

      iansSpinUnlock(&ians_lock);

      return res;
}


static struct net_device_stats *iansVadapterGetStats(struct net_device *dev)
{
      device_t *ansDev = NULL;
      struct net_device_stats *res = NULL;

      ASSERT(dev);

      iansStartBH(&ians_lock);

      ansDev = ANSGetVadapterAnsDevice(dev->name);
      if (ansDev == NULL) {
            IANS_PRINT_ERROR("there is no such ans device.\n");
            return NULL;
      }

      res =(struct net_device_stats *) ansDev->getStats(ansDev);

      iansEndBH(&ians_lock);

      return res;      
} 


// This function handles both the packet filter and the multicast list.
static void iansVadapterSetMulticastList(struct net_device *dev)
{
      MULTICAST_LIST multicastList;
      PACKET_FILTER  packetFilter;
      device_t *ansDev = NULL;
      int res;

      ASSERT(dev);

      iansStartBH(&ians_lock);

      memset( &multicastList , 0 , sizeof(MULTICAST_LIST) );


      if (dev->mc_list) {
            ASSERT(dev->mc_count);
            res = _convertOsMCListToAnsMCList(dev->mc_list, dev->mc_count, &multicastList);
            if (res != IANS_OK) {
                  iansEndBH(&ians_lock);
                  return;
            }
      }

      _convertOsPFToAnsPF(dev->flags, &packetFilter);

      ansDev = ANSGetVadapterAnsDevice(dev->name);
      if (ansDev == NULL) {
            IANS_PRINT_ERROR("there is no such ans device.\n");
            iansEndBH(&ians_lock);
            return;
      }

      res = ansDev->setMulticastList(ansDev, &multicastList, packetFilter);
      if (res != IANS_OK) {
            IANS_PRINT_ERROR("Set multicast list failed!\n");
            iansEndBH(&ians_lock);
            return;
      }

      iansEndBH(&ians_lock);
}


static int iansVadapterSetMacAddress(struct net_device *dev, void *addr)
{
      device_t *ansDev = NULL;
      sockAddr_t *sockaddr;
      int res;

      ASSERT(dev && addr);

      //iansStartBH(&ians_lock);

      ansDev = ANSGetVadapterAnsDevice(dev->name);
      if (ansDev == NULL) {
            IANS_PRINT_ERROR("there is no such ans device.\n");
            //iansEndBH(&ians_lock);
            return -EAGAIN;
      }

      sockaddr = (sockAddr_t*)addr;

      res = ansDev->setMacAddr(ansDev, sockaddr);
      if (res != IANS_OK) {
            IANS_PRINT_ERROR("Set MAC address failed!\n");
            //iansEndBH(&ians_lock);
            return -1;
      }

      //iansEndBH(&ians_lock);
      return 0;
}


static int iansVadapterDoIoctl(struct net_device *dev, struct ifreq *rq, int cmd)
{
      device_t *ansDev = NULL;
      int res;

      ASSERT(dev);      

      iansStartBH(&ians_lock);

      if ((cmd >= SIOCDEVPRIVATE) && (cmd < (SIOCDEVPRIVATE+16))) {
            // Private IOCTL's handling is undefined. reject them.
            IANS_PRINT_DEBUG("vadapter %s got a private IOCTL 0x%04X\n", dev->name, cmd);
            iansEndBH(&ians_lock);
            return 0;
      }

      ansDev = ANSGetVadapterAnsDevice(dev->name);
      if (ansDev == NULL) {
            IANS_PRINT_ERROR("there is no such ans device.\n");
            iansEndBH(&ians_lock);
            return -EAGAIN;
      }

      res = ansDev->doIoctl(ansDev, rq, cmd);

      iansEndBH(&ians_lock);

      return res;
}


static int iansVadapterChangeMtu(struct net_device *dev, int new_mtu)
{
      device_t *ansDev = NULL;
      int res;

      ASSERT(dev);

      if (new_mtu == dev->mtu) {
            //nothing to do
            return 0;
      }

      if (new_mtu < 0) {
            //must be positive
            return -EINVAL;
      }

      iansStartBH(&ians_lock);

      ansDev = ANSGetVadapterAnsDevice(dev->name);
      if (ansDev == NULL) {
            IANS_PRINT_ERROR("there is no such ans device.\n");
            iansEndBH(&ians_lock);
            return -EAGAIN;
      }

      res = ansDev->changeMtu(ansDev, new_mtu);
      if (res != IANS_OK) {
            IANS_PRINT_ERROR("Change MTU failed!\n");
            iansEndBH(&ians_lock);
            return -EINVAL;
      }

      iansEndBH(&ians_lock);
      return 0;
}


static int iansOpenReceive (struct sk_buff *skb, struct net_device *dev, struct packet_type *pt)
{
      int status = 0;
      int ansRes = IANS_OK;
      message_t msg;

      ASSERT(skb && dev);

      iansSpinLock(&ians_lock);

      if (skb->protocol != IANS_FRAME_TYPE) {
            IANS_PRINT_ERROR("Protocol of received sk_buff differs from the IANS frame type!\n");
            IANS_PRINT_ERROR("Protocol is set to %x\n", skb->protocol);
      }
      else {
		    iAnsMemSet( &msg, 0 , sizeof(message_t) );
            _convertOsMessageToAnsMessage(skb, &msg, RECEIVE_FLOW); 
            ansRes = ANSReceive(&msg, dev->name,(pPacket_t) pt);
            if (ansRes != IANS_OK)
                  status = 1;
      }

      if (status == 1) {
            dev_kfree_skb (skb);
            status = 0;
      }

      iansSpinUnlock(&ians_lock);

      return status;
}


//--------------------------- WRAPPER FUNCTIONS ------------------------------


void  iansConvertMessageToAnsMessage(pOsMessage_t osMsg, message_t *ansMsg, IANS_FLOW_MODE flow)
{
      _convertOsMessageToAnsMessage(osMsg, ansMsg, flow);
}

int iansIfReceive(device_t *ansDev, message_t *msg)
{
      struct sk_buff *skb;
      int FlowStatus = FLOW_DOESNT_EXIST;

      ASSERT(ansDev && msg);
      skb = (struct sk_buff *) GET_OS_MSG(msg);
      ASSERT(skb);

      // set to vAdapter device
      skb->dev = (struct net_device *) GET_OS_DEV(ansDev);

      ASSERT(skb->dev);

      netif_rx (skb);     // pass the packet up the stack 

      FlowStatus = FLOW_TRANSFERED;

      return( FlowStatus );
}


int iansSetMaster(device_t *ansSlave, device_t *ansMaster)
{
#ifdef LINUX_KERNEL_24X
      struct net_device *slave = (struct net_device *)GET_OS_DEV(ansSlave);
      struct net_device *master = NULL;
      BOOLEAN ioctlContext = ANSGetConfigExecuteCommand();      
      int res = IANS_OK;

      ASSERT(slave);

      if (ansMaster) //enslave
            master = (struct net_device *)GET_OS_DEV(ansMaster);

      if (!ioctlContext) {
            rtnl_lock();
      }
      if (netdev_set_master(slave, master) != 0) {
            res = (master ? IANS_FAILED_ENSLAVE_MEMBER : IANS_FAILED_RELEASE_MEMBER);
      }

      if (!ioctlContext) {
            rtnl_unlock();
      }

      return res;
#else
      return IANS_OK;
#endif
}


void iansDevPut(device_t *ansDev)
{
    struct net_device *dev = (struct net_device *) GET_OS_DEV(ansDev);

#ifdef LINUX_KERNEL_24X
      dev_put(dev);   
#endif
}

BOOLEAN iansIfStopped(device_t *ansDev)
{
    return netif_queue_stopped((struct net_device *) GET_OS_DEV(ansDev));  
}


void iansIfStop(device_t *ansDev)
{
      netif_stop_queue((struct net_device *) GET_OS_DEV(ansDev));
}

void iansIfWake(device_t *ansDev)
{
      netif_wake_queue((struct net_device *) GET_OS_DEV(ansDev));
}

void iansLock(void)
{
      iansSpinLock(&ians_lock);
}

void iansUnlock(void)
{
      iansSpinUnlock(&ians_lock);
}

void iansLockBH(void)
{
      iansStartBH(&ians_lock);
}

void iansUnlockBH(void)
{
      iansEndBH(&ians_lock);
}

void iansHotLock(BOOLEAN blockIntr, int *flags)
{
      if (blockIntr) {
            iansStartIrq(&ians_ind_lock, (*flags));
      }
      else {
            iansSpinLock(&ians_ind_lock);
      }
}

void iansHotUnlock(BOOLEAN blockIntr, int *flags)
{
      if (blockIntr) {
            iansEndIrq(&ians_ind_lock, (*flags));
      }
      else {
            iansSpinUnlock(&ians_ind_lock);
      }         
}

BOOLEAN iansIsKernelVersionGreaterThan2_4_0(void)
{
#ifdef LINUX_KERNEL_24X
      return TRUE;
#else
      return FALSE;
#endif
}

int iansHighPriority(void) {
    return TC_PRIO_CONTROL;
}

//returns kernels ticks per second
int iansGetKernelFreq(void)
{
      return HZ;
}

USHORT iansUsecsToTicks (ULONG microsecs)
{
      return (USHORT)(microsecs*0.000001*HZ);
}

BOOLEAN iansBaseDevIsOpen(device_t *ansDev)
{
      struct net_device *dev=NULL;
      ASSERT(ansDev);
      dev = (struct net_device *) GET_OS_DEV(ansDev);
      ASSERT(dev);

      if (netif_running(dev) || (dev->flags & IFF_UP))
            return TRUE;
      return FALSE;
}


ULONG iansGetDeviceBaseAddr(device_t *ansDev)
{
      struct net_device *dev=NULL;
      ASSERT(ansDev);
      dev = (struct net_device *) GET_OS_DEV(ansDev);
      ASSERT(dev);     

      return (ULONG) dev->base_addr;
}

int iansGetDevBaseAddrByName(char *devName, UINT32 *baseAddr)
{
      struct net_device *dev;

      dev = dev_get_by_name(devName);
      if (dev == NULL) {
            //IANS_PRINT_ERROR("Device %s doesn't exist!\n", devName);
            return IANS_ERROR;
      }

      *baseAddr = (UINT32)dev->base_addr;
#ifdef LINUX_KERNEL_24X
      dev_put(dev);
#endif
      return IANS_OK;
}

void iansCopyStats(pstats_t destStats, pstats_t srcStats)
{
      ASSERT(destStats && srcStats);

      destStats->rx_packets = srcStats->rx_packets;
      destStats->tx_packets = srcStats->tx_packets;
      destStats->rx_bytes = srcStats->rx_bytes;
      destStats->tx_bytes = srcStats->tx_bytes;
      destStats->rx_errors = srcStats->rx_errors;
      destStats->tx_errors = srcStats->tx_errors;
      destStats->rx_dropped = srcStats->rx_dropped;
      destStats->tx_dropped = srcStats->tx_dropped;
      destStats->multicast = srcStats->multicast;
      destStats->collisions = srcStats->collisions; 
      destStats->rx_length_errors = srcStats->rx_length_errors;
      destStats->rx_over_errors = srcStats->rx_over_errors;
      destStats->rx_crc_errors = srcStats->rx_crc_errors;
      destStats->rx_frame_errors = srcStats->rx_frame_errors;
      destStats->rx_fifo_errors = srcStats->rx_fifo_errors;
      destStats->rx_missed_errors = srcStats->rx_missed_errors;

      /* detailed tx_errors */
      destStats->tx_aborted_errors = srcStats->tx_aborted_errors;
      destStats->tx_carrier_errors = srcStats->tx_carrier_errors;
      destStats->tx_fifo_errors = srcStats->tx_fifo_errors;
      destStats->tx_heartbeat_errors = srcStats->tx_heartbeat_errors;
      destStats->tx_window_errors = srcStats->tx_window_errors;

      /* for cslip etc */
      destStats->rx_compressed = srcStats->rx_compressed;
      destStats->tx_compressed = srcStats->tx_compressed;
}

void iansAddStats(pstats_t destStats, pstats_t srcStats, BOOLEAN all)
{
      ASSERT(destStats && srcStats);

      if (all) {
            destStats->rx_packets += srcStats->rx_packets;
            destStats->tx_packets += srcStats->tx_packets;
      }
      destStats->rx_bytes += srcStats->rx_bytes;
      destStats->tx_bytes += srcStats->tx_bytes;
      destStats->rx_errors += srcStats->rx_errors;
      destStats->tx_errors += srcStats->tx_errors;
      destStats->rx_dropped += srcStats->rx_dropped;
      destStats->tx_dropped += srcStats->tx_dropped;
      destStats->multicast += srcStats->multicast;
      destStats->collisions += srcStats->collisions; 
      destStats->rx_length_errors += srcStats->rx_length_errors;
      destStats->rx_over_errors += srcStats->rx_over_errors;
      destStats->rx_crc_errors += srcStats->rx_crc_errors;
      destStats->rx_frame_errors += srcStats->rx_frame_errors;
      destStats->rx_fifo_errors += srcStats->rx_fifo_errors;
      destStats->rx_missed_errors += srcStats->rx_missed_errors;

      /* detailed tx_errors */
      destStats->tx_aborted_errors += srcStats->tx_aborted_errors;
      destStats->tx_carrier_errors += srcStats->tx_carrier_errors;
      destStats->tx_fifo_errors += srcStats->tx_fifo_errors;
      destStats->tx_heartbeat_errors += srcStats->tx_heartbeat_errors;
      destStats->tx_window_errors += srcStats->tx_window_errors;

      /* for cslip etc */
      destStats->rx_compressed += srcStats->rx_compressed;
      destStats->tx_compressed += srcStats->tx_compressed;
}

void iansSubStats(pstats_t destStats, pstats_t srcStats, BOOLEAN all)
{
      ASSERT(destStats && srcStats);

      if (all) {
            destStats->rx_packets -= srcStats->rx_packets;
            destStats->tx_packets -= srcStats->tx_packets;
      }
      destStats->rx_bytes -= srcStats->rx_bytes;
      destStats->tx_bytes -= srcStats->tx_bytes;
      destStats->rx_errors -= srcStats->rx_errors;
      destStats->tx_errors -= srcStats->tx_errors;
      destStats->rx_dropped -= srcStats->rx_dropped;
      destStats->tx_dropped -= srcStats->tx_dropped;
      destStats->multicast -= srcStats->multicast;
      destStats->collisions -= srcStats->collisions; 
      destStats->rx_length_errors -= srcStats->rx_length_errors;
      destStats->rx_over_errors -= srcStats->rx_over_errors;
      destStats->rx_crc_errors -= srcStats->rx_crc_errors;
      destStats->rx_frame_errors -= srcStats->rx_frame_errors;
      destStats->rx_fifo_errors -= srcStats->rx_fifo_errors;
      destStats->rx_missed_errors -= srcStats->rx_missed_errors;

      /* detailed tx_errors */
      destStats->tx_aborted_errors -= srcStats->tx_aborted_errors;
      destStats->tx_carrier_errors -= srcStats->tx_carrier_errors;
      destStats->tx_fifo_errors -= srcStats->tx_fifo_errors;
      destStats->tx_heartbeat_errors -= srcStats->tx_heartbeat_errors;
      destStats->tx_window_errors -= srcStats->tx_window_errors;

      /* for cslip etc */
      destStats->rx_compressed -= srcStats->rx_compressed;
      destStats->tx_compressed -= srcStats->tx_compressed;
}

int iansGetSizeofStats(void)
{
      return(int)sizeof(struct net_device_stats);
}

void iansResetStats(pstats_t stats)
{
      ASSERT(stats);
      memset((void *)(stats), 0, sizeof(struct net_device_stats));  
}

void iansSetStats(pstats_t stats, NATIVE_ULONG rx_packets, NATIVE_ULONG tx_packets)
{
      ASSERT(stats);
      stats->rx_packets = rx_packets;
      stats->tx_packets = tx_packets;
}

NATIVE_ULONG iansGetRxPacketsStats(pstats_t stats)
{
      ASSERT(stats);
      return stats->rx_packets;  
}

NATIVE_ULONG iansGetTxPacketsStats(pstats_t stats)
{
      ASSERT(stats);
      return stats->tx_packets;  
}

// ----------------------CONTROL DEVICE FUNCTIONS --------------------------

static int iansControlDoIoctl(struct net_device *dev, struct ifreq *rq, int cmd)
{
      int res = IANS_OK;
      char *data = rq->ifr_ifru.ifru_data;

      if (dev != control_g) {
            IANS_PRINT_ERROR("Got control IOCTL on non control device : dev->name %s\n",dev->name);
            return IANS_OS_ERROR;
      }

      switch (cmd) {
            
            case IANSIOCCONFIG:      
                  if (data == NULL) {
                        IANS_PRINT_ERROR("Got null data in control IOCTL");
                        res = IANS_OS_ERROR;
                        break;
                  }

                  ANSSetConfigExecuteCommand(1);
                  res = ANSDoConfigIoctl(data);
                  ANSSetConfigExecuteCommand(0);
                  break;

            case IANSIOCSERVICE:
                  ANSSetConfigExecuteCommand(1);
                  res = ANSDoServiceIoctl(data);
                  ANSSetConfigExecuteCommand(0);
                  break;

            default:
                  IANS_PRINT_DEBUG("Wrong Ioctl");
                  res = IANS_OS_ERROR;
      }

      return res;
}

static int iansControlOpen(struct net_device *dev)
{
      IANS_PRINT_DEBUG ("Got open on control device\n");
      return -EPERM;
}

static int iansControlStop(struct net_device *dev)
{
      IANS_PRINT_DEBUG ("Got stop on control device\n");
      return -EPERM;
}

static int iansControlSend(struct sk_buff *skb, struct net_device *dev)
{
      IANS_PRINT_DEBUG ("Got hard_start_xmit for control device\n");
      return 0;
}

static struct net_device_stats *iansControlGetStats(struct net_device *dev)
{
      //IANS_PRINT_DEBUG ("Got get_statistics for control device\n");
      return NULL;
}

static void iansControlSetMCList(struct net_device *dev)
{
      IANS_PRINT_DEBUG ("Got set_multicast_list for control device\n");
}

static int iansControlSetMacAddress(struct net_device *dev, void *addr)
{
      IANS_PRINT_DEBUG ("Got set_mac_address for control device\n");
      return -EPERM;
}

static int iansControlChangeMtu(struct net_device *dev, int new_mtu)
{
      IANS_PRINT_DEBUG ("Got change_mtu for control device\n");
      return -EPERM;
}

#ifdef LINUX_KERNEL_24X
static int ians_netdev_event(struct notifier_block *nb, unsigned long notification, void *ptr)
{
      int res = IANS_OK;
      struct net_device *dev = NULL;
#ifdef DEBUG
      /*
      static char* NotifyStrings[] = {
            "",
            "NETDEV_UP",
            "NETDEV_DOWN",
            "NETDEV_REBOOT",
            "NETDEV_CHANGE",
            "NETDEV_REGISTER",
            "NETDEV_UNREGISTER",
            "NETDEV_CHANGEMTU",
            "NETDEV_CHANGEADDR",
            "NETDEV_GOING_DOWN",
            "NETDEV_CHANGENAME"
      };
      */
#endif

      dev = (struct net_device*)ptr;
      if (dev && (notification>=NETDEV_UP) && (notification<=NETDEV_CHANGENAME)) {
            //IANS_PRINT_DEBUG("Got %s notification from %s\n", NotifyStrings[notification], dev->name);
            res = ANSHandleNetDevEvent(dev->name, notification);
            if (res != IANS_OK) {
                  IANS_PRINT_ERROR("couldn't handle notification");
            }
      }
      return NOTIFY_DONE;
}

static struct notifier_block ians_netdev_notifier = {
      notifier_call: ians_netdev_event
};
#endif

//--------------------------------- MODULE FUNCTIONS ---------------------------

int ians_init_module(void)
{
      int res=0;

      iansSpinLockInit(&ians_lock);
      iansSpinLockInit(&ians_ind_lock);

      res = ANSConfigCreate();
      if (res!=IANS_OK)
            return -1;

      if (( control_g = iansInitEtherdev( "ians" ) ) == NULL) {
            IANS_PRINT_ERROR ("Init etherdev failed - cannot load the IANS module.\n");
            return -1;
      }

      control_g->open               = iansControlOpen;
      control_g->stop               = iansControlStop;
      control_g->hard_start_xmit    = iansControlSend;
      control_g->get_stats          = iansControlGetStats;
      control_g->set_multicast_list = iansControlSetMCList;
      control_g->set_mac_address    = iansControlSetMacAddress;
      control_g->do_ioctl           = iansControlDoIoctl;
      control_g->change_mtu         = iansControlChangeMtu;
      control_g->flags              = 0;
      control_g->mtu                = 0;
      control_g->tx_queue_len       = 0;
#ifdef LINUX_KERNEL_22X
      control_g->start              = 0;
#endif

      //SET_MODULE_OWNER(control_g); // ??? necessary ?

      register_netdev(control_g);

#ifdef LINUX_KERNEL_24X
      register_netdevice_notifier(&ians_netdev_notifier);
#endif

      dev_add_pack(&iansPacketType);

      return 0;
}


void ians_cleanup_module(void)
{
      dev_remove_pack(&iansPacketType);

#ifdef LINUX_KERNEL_24X
      unregister_netdevice_notifier(&ians_netdev_notifier);
#endif

      unregister_netdev(control_g);

      iNCGGPMemFree( (void**)(&control_g) );

      ANSDownPhase1();
      
      iansStartBH(&ians_lock);
      ANSDownPhase2();
      iansEndBH(&ians_lock);

#ifdef MEM_DEBUG
      do {
            int i = iNCGGPMemGetAllocCount();
            if (i) {
                  IANS_PRINT_DEBUG("Allocation count = %d\n", i);
            }
      } while(0);
#endif                      
}


module_init(ians_init_module);
module_exit(ians_cleanup_module);

