/*******************************************************************************
* This software program is available to you under a choice of one of two 
* licenses. You may choose to be licensed under either the GNU General Public 
* License (GPL) Version 2, June 1991, available at 
* http://www.fsf.org/copyleft/gpl.html, or the Intel BSD + Patent License, the 
* text of which follows:
* 
* Recipient has requested a license and Intel Corporation ("Intel") is willing
* to grant a license for the software entitled Linux Intel Advanced Network
* Services (iANS) (the "Licensed Program") being provided by Intel Corporation.
* The following definitions apply to this license:
* 
* "Licensed Patents" means patent claims licensable by Intel Corporation which 
* are necessarily infringed by the use of sale of the Software alone or when 
* combined with the operating system referred to below.
* 
* "Recipient" means the party to whom Intel delivers this Software.
* 
* "Licensee" means Recipient and those third parties that receive a license to 
* any operating system available under the GNU Public License version 2.0 or 
* later.
* 
* Copyright (c) 1999 - 2002 Intel Corporation.
* All rights reserved.
* 
* The license is provided to Recipient and Recipient's Licensees under the 
* following terms.
* 
* Redistribution and use in source and binary forms of the Software, with or 
* without modification, are permitted provided that the following conditions 
* are met:
* 
* Redistributions of source code of the Software may retain the above 
* copyright notice, this list of conditions and the following disclaimer.
* 
* Redistributions in binary form of the Software may reproduce the above 
* copyright notice, this list of conditions and the following disclaimer in 
* the documentation and/or materials provided with the distribution.
* 
* Neither the name of Intel Corporation nor the names of its contributors 
* shall be used to endorse or promote products derived from this Software 
* without specific prior written permission.
* 
* Intel hereby grants Recipient and Licensees a non-exclusive, worldwide, 
* royalty-free patent license under Licensed Patents to make, use, sell, offer 
* to sell, import and otherwise transfer the Software, if any, in source code 
* and object code form. This license shall include changes to the Software 
* that are error corrections or other minor changes to the Software that do 
* not add functionality or features when the Software is incorporated in any 
* version of an operating system that has been distributed under the GNU 
* General Public License 2.0 or later. This patent license shall apply to the 
* combination of the Software and any operating system licensed under the GNU 
* Public License version 2.0 or later if, at the time Intel provides the 
* Software to Recipient, such addition of the Software to the then publicly 
* available versions of such operating systems available under the GNU Public 
* License version 2.0 or later (whether in gold, beta or alpha form) causes 
* such combination to be covered by the Licensed Patents. The patent license 
* shall not apply to any other combinations which include the Software. NO 
* hardware per se is licensed hereunder.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MECHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR IT CONTRIBUTORS BE LIABLE FOR ANY 
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
* ANY LOSS OF USE; DATA, OR PROFITS; OR BUSINESS INTERUPTION) HOWEVER CAUSED 
* AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY OR 
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/
 /***************************************************************************
 **                                                                        **
 ** INTEL CORPORATION                                                      **
 **                                                                        **
 ** This software is supplied under the terms of the license included      **
 ** above.  All use of this software must be in accordance with the terms  **
 ** of that license.                                                       **
 **                                                                        **
 **  Module Name:                                                          **
 **    incg_log.c                                                          **
 **                                                                        **
 **  Abstract:                                                             **
 **    This module prints log messages for iANS.                           **
 **                                                                        **
 **  Environment:                                                          **
 **    Kernel Mode (linux 2.2.x, 2.4.x)                                    **
 **                                                                        **
 ***************************************************************************/

#include <stdarg.h>

#include "incg_utils.h"
//#include "incg_log.h"


void iNCGPrintArgs (const char * fmt,  ...)
{

	va_list ap;
	const char *p;
	char *sval;
	int ival, i;
	ULONG ulval;
	UINT32 uival;
	char arg_buffer[MAX_ARG_LENGTH];
	char message_buffer[MAX_LINE_LENGTH+5]; /* add 5 for CR and LF and stuff */
	int msg_buf_idx=0;
	int rem;

	message_buffer[msg_buf_idx++]=' ';	 /* set the first WORD to be '  ' so that */
	message_buffer[msg_buf_idx++]=' ';	 /* LINUX print will put this as the number */
        message_buffer[msg_buf_idx++]='A';       /* for ALL C++ errors. Else it will just  */
        message_buffer[msg_buf_idx++]='N';       /* put some random value which is not  */        
        message_buffer[msg_buf_idx++]='S';       /* uniform in look. */
        message_buffer[msg_buf_idx++]=':';
        message_buffer[msg_buf_idx++]=' ';        

	/* first parameter is format */
	va_start(ap, fmt);

	for (p=fmt; *p; p++)
	{
		if (*p != '%')
		{

			/* check there's enough room for one char and a null terminator */
			if (msg_buf_idx > MAX_LINE_LENGTH-2)
				break;

			message_buffer[msg_buf_idx++]=(*p);
			message_buffer[msg_buf_idx]='\0';
		
			continue;
		}
		
		switch(*++p)
		{
			case 'd':
				ival=va_arg(ap, int);
				iAnsItoa(ival, (char*)arg_buffer);
				arg_buffer[iAnsStrLen(arg_buffer)]='\0';
				
				for (i=0;i<iAnsStrLen(arg_buffer);i++)
				{
					/* check there's enough room for one char and a null terminator */
					if (msg_buf_idx > MAX_LINE_LENGTH-2)
						break;
					message_buffer[msg_buf_idx++]=arg_buffer[i];
					message_buffer[msg_buf_idx]='\0';
				}
				
				break;

			case 'l':
				ulval=va_arg(ap, ULONG);
				iAnsUltoa(ulval, (char*)arg_buffer, 10);
				arg_buffer[iAnsStrLen(arg_buffer)]='\0';
				
				for (i=0;i<iAnsStrLen(arg_buffer);i++)
				{
					/* check there's enough room for one char and a null terminator */
					if (msg_buf_idx > MAX_LINE_LENGTH-2)
						break;
					message_buffer[msg_buf_idx++]=arg_buffer[i];
					message_buffer[msg_buf_idx]='\0';
				}
				
				break;

			case 's':
				for (sval=va_arg(ap, char*);*sval;sval++)
				{
					/* check there's enough room for one char and a null terminator */
					if (msg_buf_idx > MAX_LINE_LENGTH-2)
						break;
					message_buffer[msg_buf_idx++]=(*sval);
					message_buffer[msg_buf_idx]='\0';
				}
					
				break;


			case 'x':
			case 'X':
			case 'W':	/* prints DWORDS */
				/* check there's enough room for the DWORD and a null terminator */
				if (msg_buf_idx + 8 > MAX_LINE_LENGTH-1)
					break;

				/* zero the string space for the DWORD */
				for (i=0;i<8;i++)
					message_buffer[msg_buf_idx + i]='0';

				msg_buf_idx +=7;
				
				uival=va_arg(ap, UINT32);
				
				/* Starting from the end of the DWORD, print every character */
				for (i=0;i<8;i++)
				{
					rem = (int) (uival % 16);

					if (rem < 10)
						message_buffer[msg_buf_idx] = rem + '0';
					else
						message_buffer[msg_buf_idx] = (rem - 10) + 'a';
												
					msg_buf_idx--;
					uival = (int)(uival/16);
				}
				/* msg_buf_idx now points to one location before the location we started at, and we checked
					already that we have enough room */
				msg_buf_idx +=9;
				message_buffer[msg_buf_idx]='\0';

				break;

			default:
				return;

		}

	}
	
	va_end(ap);

	/* there's enough room for the CR/LF because we allocated MAX_LINE_LENGTH+5 and we checked that we didn't
		put more than MAX_LINE_LENGTH characters in the string */
	message_buffer[msg_buf_idx++]=10;
	message_buffer[msg_buf_idx]='\0';	
	iAnsKernelPrint (INGC_NO_LEVEL, message_buffer); /* Cont to prevent extra \n */

}





