/*******************************************************************************
* This software program is available to you under a choice of one of two 
* licenses. You may choose to be licensed under either the GNU General Public 
* License (GPL) Version 2, June 1991, available at 
* http://www.fsf.org/copyleft/gpl.html, or the Intel BSD + Patent License, the 
* text of which follows:
* 
* Recipient has requested a license and Intel Corporation ("Intel") is willing
* to grant a license for the software entitled Linux Intel Advanced Network
* Services (iANS) (the "Licensed Program") being provided by Intel Corporation.
* The following definitions apply to this license:
* 
* "Licensed Patents" means patent claims licensable by Intel Corporation which 
* are necessarily infringed by the use of sale of the Software alone or when 
* combined with the operating system referred to below.
* 
* "Recipient" means the party to whom Intel delivers this Software.
* 
* "Licensee" means Recipient and those third parties that receive a license to 
* any operating system available under the GNU Public License version 2.0 or 
* later.
* 
* Copyright (c) 1999 - 2002 Intel Corporation.
* All rights reserved.
* 
* The license is provided to Recipient and Recipient's Licensees under the 
* following terms.
* 
* Redistribution and use in source and binary forms of the Software, with or 
* without modification, are permitted provided that the following conditions 
* are met:
* 
* Redistributions of source code of the Software may retain the above 
* copyright notice, this list of conditions and the following disclaimer.
* 
* Redistributions in binary form of the Software may reproduce the above 
* copyright notice, this list of conditions and the following disclaimer in 
* the documentation and/or materials provided with the distribution.
* 
* Neither the name of Intel Corporation nor the names of its contributors 
* shall be used to endorse or promote products derived from this Software 
* without specific prior written permission.
* 
* Intel hereby grants Recipient and Licensees a non-exclusive, worldwide, 
* royalty-free patent license under Licensed Patents to make, use, sell, offer 
* to sell, import and otherwise transfer the Software, if any, in source code 
* and object code form. This license shall include changes to the Software 
* that are error corrections or other minor changes to the Software that do 
* not add functionality or features when the Software is incorporated in any 
* version of an operating system that has been distributed under the GNU 
* General Public License 2.0 or later. This patent license shall apply to the 
* combination of the Software and any operating system licensed under the GNU 
* Public License version 2.0 or later if, at the time Intel provides the 
* Software to Recipient, such addition of the Software to the then publicly 
* available versions of such operating systems available under the GNU Public 
* License version 2.0 or later (whether in gold, beta or alpha form) causes 
* such combination to be covered by the Licensed Patents. The patent license 
* shall not apply to any other combinations which include the Software. NO 
* hardware per se is licensed hereunder.
* 
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
* IMPLIED WARRANTIES OF MECHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR IT CONTRIBUTORS BE LIABLE FOR ANY 
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
* (INCLUDING, BUT NOT LIMITED, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
* ANY LOSS OF USE; DATA, OR PROFITS; OR BUSINESS INTERUPTION) HOWEVER CAUSED 
* AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY OR 
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/
 /***************************************************************************
 **                                                                        **
 ** INTEL CORPORATION                                                      **
 **                                                                        **
 ** This software is supplied under the terms of the license included      **
 ** above.  All use of this software must be in accordance with the terms  **
 ** of that license.                                                       **
 **                                                                        **
 **  THIS FILE IS INCLUDED BY ians_core.o. HENCE, IT SHOULD NOT BE         **
 **  MODIFIED!                                                             **
 **                                                                        **
 **  Module Name:                                                          ** 
 **    incg_net.h                                                          **
 **                                                                        **
 **  Abstract:                                                             **
 **    This module defines iANS networking (mac address, multicast list,   **
 **    packet filter etc.).                                                **
 **                                                                        **
 **  Environment:                                                          **
 **    Kernel Mode (linux 2.2.x, 2.4.x)                                    **
 **                                                                        **
 ****************************************************************************/




#ifndef _INCG_NET_H_
#define _INCG_NET_H_
/* This file should be split to ethernet and general networking.
 * We did not do it yet necause it is too small to justify the
 * split */


#include "incg_defs.h"

#define MAC_ADDR_LEN 6

#pragma pack(1) 
typedef struct _MACADDR {
	UCHAR	MacAddr[MAC_ADDR_LEN];

} MAC_ADDR, *PMAC_ADDR;
#pragma pack() 

#define DUMMY_MAC_ADDR  { 0, 0, 0, 0, 0, 0 }



 /* This macro will compare network addresses.
  *
  * A - Is a network address.
  *
  * B - Is a network address.
  *
  * Result - The result of comparing two network address.
  *
  * Result != 0 Implies inequality.
  * Result == 0 Implies equality.
  */

  //We cannot make any assumptions on the alignment of _A and _B
  //so we must compare them byte by byte.
#define ETH_COMPARE_NETWORK_ADDRESSES_EQ(_A,_B, _Result)		\
   {							    \
	if ((*(UCHAR *)&(_A)[5] == *(UCHAR *)&(_B)[5]) &&   \
	      (*(UCHAR *)&(_A)[4] == *(UCHAR *)&(_B)[4]) && \
	      (*(UCHAR *)&(_A)[3] == *(UCHAR *)&(_B)[3]) && \
	      (*(UCHAR *)&(_A)[2] == *(UCHAR *)&(_B)[2]) && \
	      (*(UCHAR *)&(_A)[1] == *(UCHAR *)&(_B)[1]) && \
	      (*(UCHAR *)(_A) == *(UCHAR *)(_B))	    \
	   )				       		    \
	   {						    \
		*(_Result) = 0;				    \
	   }						    \
       else			   			    \
	   {						    \
		*(_Result) = 1;				    \
	   }						    \
   }
   

/* Compares 2 ethernet addresses. 
 * returns 0 if equal, otherwise returns result !=0.
 */
extern inline int
iNCGAddrCompareEthAddresses (MAC_ADDR Addr1, MAC_ADDR Addr2)
{
	int Result;
	ETH_COMPARE_NETWORK_ADDRESSES_EQ(Addr1.MacAddr, Addr2.MacAddr, &Result);
	return (Result);
}


#define iNCGMacAddrCopy(ToAddr, FromAddr) iAnsMemCpy( ((void*)ToAddr), ((void*)FromAddr), MAC_ADDR_LEN );

/*-------------------------------------------------------------------------
   PACKET_FILTER
  -------------------------------------------------------------------------*/											
typedef ULONG	 PACKET_FILTER, *PPACKET_FILTER;

#define PF_PROMISCUOUS_BIT 0x00000001  /*  If this bit is set in the packet 
										   filter, set promisuous mode ON */

#define PF_ALLMCA_BIT  0x00000002 /*  If this bit is set in the packet 
										   filter, send MACIOC_SETALLMCA.
										   else send MACIOC_DELALLMCA */
#define DUMMY_PACKET_FILTER 0 

/*-------------------------------------------------------------------------
   MULTICAST_LIST
  -------------------------------------------------------------------------*/ 
 
typedef struct _MULTICAST_ENTRY 
{
	struct _MULTICAST_ENTRY    *pNextEntry;
	MAC_ADDR            MacAddr;
    USHORT              dmiAddrLen;
    USHORT              dmiUsers;
	USHORT              dmiGusers;
} MULTICAST_ENTRY, *PMULTICAST_ENTRY;
	
typedef struct _MULTICAST_LIST 
{
	PMULTICAST_ENTRY    pAddressesList;				
	UINT                NumAddresses;   /* List size */
} MULTICAST_LIST, *PMULTICAST_LIST;

#define DUMMY_MULTICAST_LIST { NULL,/* AddressesList */ 0     /* NumAddresses */ }

extern MULTICAST_LIST dummyMulticastLIst;

#endif /*_INCG_NET_H_ */


